local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local UIUtil = import('/lua/modules/ui/uiutil.lua')
local Button = import('/lua/modules/maui/button.lua').Button
local Group = import('/lua/modules/maui/group.lua').Group
local Grid = import('/lua/modules/maui/grid.lua').Grid
local GameCommon = import('/lua/modules/ui/game/gamecommon.lua')
local Tooltip = import('/lua/modules/ui/game/tooltip.lua')
local TooltipInfo = import('/lua/modules/ui/help/tooltips.lua')
local Prefs = import('/lua/modules/ui/prefs.lua')
local Keymapping = import('/lua/keymap/defaultKeyMap.lua').defaultKeyMap
local Checkbox = import('/lua/modules/maui/checkbox.lua').Checkbox
local Orders = import('/lua/modules/ui/game/orders.lua').controls


local gameParent = false
local controlClusterGroup = false
local statusClusterGroup = false
local mapGroup = false
local mfdControl = false
local ordersControl = false
local queueControl = false
local oldHandleEvent = false



function CreateRadialBG(parent)
	RadialBG = Bitmap(parent, '/mods/radialmenu/textures/radial_backgrounds/radialbg_' .. Faction .. '.dds')
	RadialBG:SetAlpha(0.5)
	RadialBG.Depth:Set(1000)
	RadialBG.Left:Set(400);
	RadialBG.Top:Set(400);
end


controls = {
    mouseoverDisplay = false,
    orderButtonGrid = false,
    bg = false,
    orderGlow = false,
    controlClusterGroup = false,
    mfdControl = false,
}

factionTable = {
    [0] = 'UEF',
	[1] = 'AEON',
	[2] = 'CYBRAN',
}


local layoutVar = false
local glowThread = false


local function CreateOrderGlow(parent)
    controls.orderGlow = Bitmap(parent, UIUtil.UIFile('/game/orders/glow-02_bmp.dds'))
    LayoutHelpers.AtCenterIn(controls.orderGlow, parent)
    controls.orderGlow:SetAlpha(0.0)
    controls.orderGlow:DisableHitTest()
    controls.orderGlow:SetNeedsFrameUpdate(true)
    local alpha = 0.0
    local incriment = true
    controls.orderGlow.OnFrame = function(self, deltaTime)
        if incriment then
            alpha = alpha + (deltaTime * 0.9)
        else
            alpha = alpha - (deltaTime * 0.9)
        end
        if alpha < 0 then
            alpha = 0.0
            incriment = true
        end
        if alpha > .3 then
            alpha = .3
            incriment = false
        end
        controls.orderGlow:SetAlpha(alpha)
    end     
end

local function CreateAutoBuildEffect(parent)
    local glow = Bitmap(parent, UIUtil.UIFile('/game/orders/glow-02_bmp.dds'))
    LayoutHelpers.AtCenterIn(glow, parent)
    glow:SetAlpha(0.0)
    glow:DisableHitTest()
    glow:SetNeedsFrameUpdate(true)
    glow.alpha = 0.0
    glow.incriment = true
    glow.OnFrame = function(self, deltaTime)
        if self.incriment then
            self.alpha = self.alpha + (deltaTime * .35)
        else
            self.alpha = self.alpha - (deltaTime * .35)
        end
        if self.alpha < 0 then
            self.alpha = 0.0
            self.incriment = true
        end
        if self.alpha > .4 then
            self.alpha = .4
            self.incriment = false
        end
        self:SetAlpha(self.alpha)
    end   
    return glow  
end


function CreateMouseoverDisplay(parent, ID)
    if controls.mouseoverDisplay then
        controls.mouseoverDisplay:Destroy()
        controls.mouseoverDisplay = false
    end
    
    if not Prefs.GetOption('tooltips') then return end
    
    local createDelay = Prefs.GetOption('tooltip_delay') or 0
    
    local text = TooltipInfo['Tooltips'][ID]['title'] or ID
    local desc = TooltipInfo['Tooltips'][ID]['description'] or ID
    
    if TooltipInfo['Tooltips'][ID]['keyID'] and TooltipInfo['Tooltips'][ID]['keyID'] != "" then
        for i, v in Keymapping do
            if v.stringkey == TooltipInfo['Tooltips'][ID]['keyID'] then
                text = LOCF("%s (%s)", text, i)
                break
            end
        end
    end
    
    controls.mouseoverDisplay = Tooltip.CreateExtendedToolTip(parent, text, desc)
    local Frame = GetFrame(0)
    controls.mouseoverDisplay.Bottom:Set(parent.Top)
    if (parent.Left() + (parent.Width() / 2)) - (controls.mouseoverDisplay.Width() / 2) < 0 then
        controls.mouseoverDisplay.Left:Set(4)
    elseif (parent.Right() - (parent.Width() / 2)) + (controls.mouseoverDisplay.Width() / 2) > Frame.Right() then
        controls.mouseoverDisplay.Right:Set(function() return Frame.Right() - 4 end)
    else
        LayoutHelpers.AtHorizontalCenterIn(controls.mouseoverDisplay, parent)
    end
    
    local alpha = 0.0
    controls.mouseoverDisplay:SetAlpha(alpha, true)
    local mdThread = ForkThread(function()
        WaitSeconds(createDelay)
        while alpha <= 1.0 do
            controls.mouseoverDisplay:SetAlpha(alpha, true)
            alpha = alpha + 0.1
            WaitSeconds(0.01)
        end
    end)

    controls.mouseoverDisplay.OnDestroy = function(self)
        KillThread(mdThread)  
    end
	
end

-- local logic data
local orderButtonMap = false
local currentSelection = false


local function GetOrderBitmapNames(bitmapId)
    if bitmapId == nil then
        LOG("Error - nil bitmap passed to GetOrderBitmapNames")
        bitmapId = "basic-empty"    --TODO do I really want to default it?
    end
    
    local button_prefix = "/game/orders/" .. bitmapId .. "_btn_"
    return UIUtil.SkinnableFile(button_prefix .. "up.dds")
        ,  UIUtil.SkinnableFile(button_prefix .. "up_sel.dds")
        ,  UIUtil.SkinnableFile(button_prefix .. "over.dds")
        ,  UIUtil.SkinnableFile(button_prefix .. "over_sel.dds")
        ,  UIUtil.SkinnableFile(button_prefix .. "dis.dds")
        ,  UIUtil.SkinnableFile(button_prefix .. "dis_sel.dds")
        , "UI_Action_MouseDown", "UI_Action_Rollover"   -- sets click and rollover cues
end

-- used by most orders, which start and stop a command mode, so they toggle on when pressed
-- and toggle off when done
local function StandardOrderBehavior(self, modifiers)
    import('/lua/modules/ui/game/commandmode.lua').StartCommandMode("order", self._order)
	RadialBG:Hide()
end

-- used by orders that happen immediately and don't change the command mode (ie the stop button)
local function MomentaryOrderBehavior(self, modifiers)
    IssueCommand(GetUnitCommandFromCommandCap(self._order))
    RadialBG:Hide()
end

-- used by things that build weapons, etc
local function BuildOrderBehavior(self, modifiers)
    if modifiers.Left then
	    IssueCommand(GetUnitCommandFromCommandCap(self._order))
    elseif modifiers.Right then
	    self:ToggleCheck()
        if self:IsChecked() then
            self._curHelpText = self._data.helpText .. "_auto"
            self.autoBuildEffect = CreateAutoBuildEffect(self)
        else
            self._curHelpText = self._data.helpText
            self.autoBuildEffect:Destroy()
        end
        if controls.mouseoverDisplay.text then
            controls.mouseoverDisplay.text:SetText(self._curHelpText)
        end
	    SetAutoMode(currentSelection, self:IsChecked())
    end
	RadialBG:Hide()
end

local function BuildInitFunction(control, unitList)
    local isAutoMode = GetIsAutoMode(unitList)
    control:SetCheck(isAutoMode)
    if isAutoMode then
        control._curHelpText = control._data.helpText .. "_auto"
        control.autoBuildEffect = CreateAutoBuildEffect(control)
    else
        control._curHelpText = control._data.helpText
    end
end

-- some toggleable abilities need reverse semantics.
local function CheckReverseSemantics(scriptBit)
    if scriptBit == 0 then # shields
        return true
    end
    
    return false
end

local function ScriptButtonOrderBehavior(self, modifiers)
    local state = self:IsChecked()
    if CheckReverseSemantics(self._data.extraInfo) then
        state = not state
    end
	ToggleScriptBit(currentSelection, self._data.extraInfo, state)
	Checkbox.OnClick(self)
    if controls.mouseoverDisplay.text then
        controls.mouseoverDisplay.text:SetText(self._curHelpText)
    end
	RadialBG:Hide()
end

local function ScriptButtonInitFunction(control, unitList)
    local result = GetScriptBit(unitList, control._data.extraInfo)
    if CheckReverseSemantics(control._data.extraInfo) then
        result = not result
    end
    control:SetCheck(result)
end

local retaliateStateInfo = {
    [-1] = {bitmap = 'stand-ground',    helpText = "mode_mixed",},
    [0] = { bitmap = 'return-fire',     helpText = "mode_return_fire",},
    [1] = { bitmap = 'hold-fire',       helpText = "mode_hold",},
}

local function RetaliateOrderBehavior(self, modifiers)
	ToggleFireState(currentSelection, self._toggleState)
	
	self._toggleState = self._toggleState + 1
	if self._toggleState >= 2 then
	    self._toggleState = 0
	end
	
    if not retaliateStateInfo[self._toggleState] then
        LOG("Error: orders.lua - invalid toggle state: ", tostring(self._toggleState))
    end
    self:SetNewTextures(GetOrderBitmapNames(retaliateStateInfo[self._toggleState].bitmap))
    self._curHelpText = retaliateStateInfo[self._toggleState].helpText
    if controls.mouseoverDisplay.text then
        controls.mouseoverDisplay.text:SetText(self._curHelpText)
    end
	RadialBG:Hide()
end

local function RetaliateInitFunction(control, unitList)
    control._toggleState = GetFireState(unitList)
    if not retaliateStateInfo[control._toggleState] then
        LOG("Error: orders.lua - invalid toggle state: ", tostring(self._toggleState))
    end
    control:SetNewTextures(GetOrderBitmapNames(retaliateStateInfo[control._toggleState].bitmap))
    control._curHelpText = retaliateStateInfo[control._toggleState].helpText
end

function CycleRetaliateStateUp()
    local currentFireState = GetFireState(currentSelection)
    if currentFireState > 2 then
        currentFireState = 0
    end
    ToggleFireState(currentSelection, currentFireState)
end

function EnterOverchargeMode()
    local econData = GetEconomyTotals()
    if not currentSelection[1] or currentSelection[1]:IsDead() then return end
    local bp = currentSelection[1]:GetBlueprint()
    local overchargeLevel = 0
    local overchargeFound = false
    for index, weapon in bp.Weapon do
        if weapon.OverChargeWeapon then
            overchargeLevel = weapon.EnergyRequired
            overchargeFound = true
            break
        end
    end
    if overchargeFound then
        if overchargeLevel > 0 and econData["stored"]["ENERGY"] > overchargeLevel then
            ConExecute('StartCommandMode order RULEUCC_Overcharge')
        end
    end
end

local function OverChargeFrame(self, deltaTime)
    if deltaTime then
        local econData = GetEconomyTotals()
        local bp = currentSelection[1]:GetBlueprint()
        local overchargeLevel = 0
        for index, weapon in bp.Weapon do
            if weapon.OverChargeWeapon then
                overchargeLevel = weapon.EnergyRequired
                break
            end
        end
        if overchargeLevel > 0 then
            if econData["stored"]["ENERGY"] > overchargeLevel then
                if self:IsDisabled() then
                    self:Enable()
                end
            else
                if not self:IsDisabled() then
                    self:Disable()
                end
            end
        else
            self:SetNeedsFrameUpdate(false)
        end
    end
end

local standardOrdersTable = {
    -- Common rules
    RULEUCC_Move = {                helpText = "move",          bitmapId = 'move',   			preferredSlot = 3,            behavior = StandardOrderBehavior, disabled = true, },
    RULEUCC_Attack = {              helpText = "attack",        bitmapId = 'attack',     		preferredSlot = 1,            behavior = StandardOrderBehavior, disabled = true, },
    RULEUCC_Patrol = {              helpText = "patrol",        bitmapId = 'patrol',    	    preferredSlot = 2,            behavior = StandardOrderBehavior, disabled = true, },
    RULEUCC_Stop = {                helpText = "stop",          bitmapId = 'stop',     	        preferredSlot = 6,            behavior = MomentaryOrderBehavior, disabled = true, },
    RULEUCC_Guard = {               helpText = "assist",        bitmapId = 'guard',      	    preferredSlot = 5,            behavior = StandardOrderBehavior, disabled = true, },
    RULEUCC_RetaliateToggle = {     helpText = "mode",          bitmapId = 'stand-ground',     	preferredSlot = 4,            behavior = RetaliateOrderBehavior, disabled = true, initialStateFunc = RetaliateInitFunction, },

    -- Unit specific rules
    RULEUCC_SiloBuildTactical = {   helpText = "build_tactical",bitmapId = 'silo-build-tactical',        preferredSlot = 7,  behavior = BuildOrderBehavior,          initialStateFunc = BuildInitFunction,},
    RULEUCC_SiloBuildNuke = {       helpText = "build_nuke",    bitmapId = 'silo-build-nuke',            preferredSlot = 7,  behavior = BuildOrderBehavior,          initialStateFunc = BuildInitFunction,},
    RULEUCC_Overcharge = {          helpText = "overcharge",    bitmapId = 'overcharge',                 preferredSlot = 7,  behavior = StandardOrderBehavior,       onframe = OverChargeFrame},
    RULEUCC_SpecialAction = {       helpText = "special_action",bitmapId = 'overcharge',            	 preferredSlot = 7,  behavior = StandardOrderBehavior,},
    RULEUCC_Transport = {           helpText = "transport",     bitmapId = 'unload',                     preferredSlot = 8,  behavior = StandardOrderBehavior, },
    RULEUCC_Nuke = {                helpText = "fire_nuke",     bitmapId = 'launch-nuke',                preferredSlot = 9,  behavior = StandardOrderBehavior, },
    RULEUCC_Tactical = {            helpText = "fire_tactical", bitmapId = 'launch-tactical',     	 	 preferredSlot = 9,  behavior = StandardOrderBehavior, },
    RULEUCC_Teleport = {            helpText = "teleport",      bitmapId = 'teleport',        	  	     preferredSlot = 9,  behavior = StandardOrderBehavior, },
    RULEUCC_Ferry = {               helpText = "ferry",         bitmapId = 'ferry',              		 preferredSlot = 9,  behavior = StandardOrderBehavior, },
    RULEUCC_Sacrifice = {           helpText = "sacrifice",     bitmapId = 'sacrifice',                  preferredSlot = 9,  behavior = StandardOrderBehavior,},
    RULEUCC_Dive = {                helpText = "dive",          bitmapId = 'dive',                       preferredSlot = 10, behavior = MomentaryOrderBehavior,},   
    RULEUCC_Reclaim = {             helpText = "reclaim",       bitmapId = 'reclaim',                    preferredSlot = 10, behavior = StandardOrderBehavior, },
    RULEUCC_Capture = {             helpText = "capture",       bitmapId = 'convert',                    preferredSlot = 11, behavior = StandardOrderBehavior,},
    RULEUCC_Repair = {              helpText = "repair",        bitmapId = 'repair',                     preferredSlot = 12, behavior = StandardOrderBehavior,},

    -- Unit toggle rules
    RULEUCC_ShieldToggle = {        helpText = "toggle_shield",     bitmapId = 'shield',                 preferredSlot = 7,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 0,},
    RULEUCC_WeaponToggle = {        helpText = "toggle_weapon",     bitmapId = 'toggle-weapon-direct',   preferredSlot = 7,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 1,},    
    RULEUCC_JammingToggle = {       helpText = "toggle_jamming",    bitmapId = 'jamming',                preferredSlot = 8,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 2,},
    RULEUCC_IntelToggle = {         helpText = "toggle_intel",      bitmapId = 'intel',                  preferredSlot = 8,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 3,},
    RULEUCC_ProductionToggle = {    helpText = "toggle_production", bitmapId = 'production',             preferredSlot = 9,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 4,},
    RULEUCC_StealthToggle = {       helpText = "toggle_stealth",    bitmapId = 'stealth',                preferredSlot = 9,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 5,},
    RULEUCC_GenericToggle = {       helpText = "toggle_generic",    bitmapId = 'production',             preferredSlot = 10, behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 6,},
    RULEUCC_SpecialToggle = {       helpText = "toggle_special",    bitmapId = 'activate-weapon',        preferredSlot = 11, behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 7,},
    RULEUCC_CloakToggle = {         helpText = "toggle_cloak",      bitmapId = 'intel-counter',          preferredSlot = 11, behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 8,},
}

local commonOrders = {
    RULEUCC_Move = true,
    RULEUCC_Attack = true,
    RULEUCC_Patrol = true,
    RULEUCC_Stop = true,
    RULEUCC_Guard = true, 
    RULEUCC_RetaliateToggle = true, 
}

-- x, -y position for each slot relative to the radialbg center
local slotLayout = {
    {0, -42},
    {45, -24},
	{45, 24},
	{0, 42},
	{-45, 24},
	{-45, -24},
    {0, -42},
    {45, -24},
	{45, 24},
	{0, 42},
	{-45, 24},
	{-45, -24},
}

local currentOrders = {}

local function AddOrder(orderInfo, slot, parent)
    local button = Checkbox(parent, GetOrderBitmapNames(orderInfo.bitmapId))

    button:SetAlpha(0.75)
    -- set the info in to the data member for retrieval
    button._data = orderInfo
    
    -- set disabled state if needed
    if orderInfo.disabled then
        button:Disable()
    end
    
    -- set up initial help text
    button._curHelpText = orderInfo.helpText

    -- set up click handler
    button.OnClick = orderInfo.behavior
    
    if orderInfo.onframe then
        button.EnableEffect = Bitmap(button, UIUtil.UIFile('/game/orders/glow-02_bmp.dds'))
        LayoutHelpers.AtCenterIn(button.EnableEffect, button)
        button.EnableEffect:DisableHitTest()
        button.EnableEffect:SetAlpha(0)
        button.EnableEffect.Incrimenting = false
        button.EnableEffect.OnFrame = function(self, deltatime)
            local alpha
            if self.Incrimenting then
                alpha = self.Alpha + (deltatime * 2)
                if alpha > 1 then
                    alpha = 1
                    self.Incrimenting = false
                end
            else
                alpha = self.Alpha - (deltatime * 2)
                if alpha < 0 then
                    alpha = 0
                    self.Incrimenting = true
                end
            end
            self.Height:Set(function() return button.Height() + (button.Height() * alpha*.5) end)
            self.Width:Set(function() return button.Height() + (button.Height() * alpha*.5) end)
            self.Alpha = alpha
            self:SetAlpha(alpha*.45)
        end
        button:SetNeedsFrameUpdate(true)
        button.OnFrame = orderInfo.onframe
        button.OnEnable = function(self)
            self.EnableEffect:SetNeedsFrameUpdate(true)
            self.EnableEffect.Incrimenting = false
            self.EnableEffect:SetAlpha(1)
            self.EnableEffect.Alpha = 1
            Checkbox.OnEnable(self)
        end
        button.OnDisable = function(self)
            self.EnableEffect:SetNeedsFrameUpdate(false)
            self.EnableEffect:SetAlpha(0)
            Checkbox.OnDisable(self)
        end
    end
	
	
    -- set up tooltips
    button.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            if controls.orderGlow then
                controls.orderGlow:Destroy()
                controls.orderGlow = false
            end                
            CreateMouseoverDisplay(self, self._curHelpText, 1)
            glowThread = CreateOrderGlow(self)
        elseif event.Type == 'MouseExit' then
            if controls.mouseoverDisplay then
                controls.mouseoverDisplay:Destroy()
                controls.mouseoverDisplay = false
            end
            if controls.orderGlow then
                controls.orderGlow:Destroy()
                controls.orderGlow = false
                KillThread(glowThread)
            end
        end
        Checkbox.HandleEvent(self, event)
    end
    Position = slotLayout[slot]
	button.Depth:Set(parent.Depth)
 	LayoutHelpers.AtCenterIn(button, parent, Position[2], Position[1])      

    return button
end


local function CreateButtons(availableOrders)
    for key in commonOrders do
	    local orderInfo = standardOrdersTable[key]
		local orderButton = AddOrder(orderInfo, orderInfo.preferredSlot, commonGroup)
		orderButton._order = key
		
        if orderInfo.initialStateFunc then
            orderInfo.initialStateFunc(orderButton, currentSelection)
        end

	    orderButtonMap[key] = orderButton
    end
    for index, availOrder in currentOrders do
        if not standardOrdersTable[availOrder] then continue end   -- skip any orders we don't have in our table
        if commonOrders[availOrder] then
            local ck = orderButtonMap[availOrder]
            ck:Enable()
        end
    end
end

local function CreateAltButtons(availableOrders)
    -- determine what slots to put alt orders
    -- we first want a table of slots we want to fill, and what orders want to fill them
    local desiredSlot = {}
    for index, availOrder in availableOrders do
        if standardOrdersTable[availOrder] then 
            local preferredSlot = standardOrdersTable[availOrder].preferredSlot
            if not desiredSlot[preferredSlot] then
                desiredSlot[preferredSlot] = {}
            end
            table.insert(desiredSlot[preferredSlot], availOrder)
		end
    end
	
    -- now go through that table and determine what doesn't fit and look for slots that are empty
    -- since this is only alt orders, just deal with slots 7-12
    local orderInSlot = {}
    
    -- go through first time and add all the first entries to their preferred slot
    for slot = 7, 12 do
        if desiredSlot[slot] then
            orderInSlot[slot] = desiredSlot[slot][1]
        end
    end

    -- now put any additional entries wherever they will fit
    for slot = 7, 12 do
        if desiredSlot[slot] and table.getn(desiredSlot[slot]) > 1 then
            for index, item in desiredSlot[slot] do
                if index > 1 then
                    local foundFreeSlot = false
                    for newSlot = 7, 12 do
                        if not orderInSlot[newSlot] then
                            orderInSlot[newSlot] = item
                            foundFreeSlot = true
                            break
                        end
                    end
                    if not foundFreeSlot then
                        WARN("No free slot for order: " .. item)
                        -- could break here, but don't, then you'll know how many extra orders you have
                    end
                end
            end
        end
    end

    -- now map it the other direction so it's order to slot
    local slotForOrder = {}
    for slot, order in orderInSlot do
        slotForOrder[order] = slot
    end
    
    -- create the alt order buttons
    for index, availOrder in availableOrders do
	    if slotForOrder[availOrder] then
    		if not standardOrdersTable[availOrder] then continue end   -- skip any orders we don't have in our table
	    	if not commonOrders[availOrder] then
		    	local orderInfo = standardOrdersTable[availOrder]
			    orderInfo.disabled = false
    			local orderButton = AddOrder(orderInfo, slotForOrder[availOrder], altGroup)
	    		orderButton._order = availOrder
		    	if orderInfo.initialStateFunc then
			    	orderInfo.initialStateFunc(orderButton, currentSelection)
    			end
		
	    		orderButtonMap[availOrder] = orderButton
			end
		end
    end
end


# Called from gamemain.lua
function SetAvailableOrders(availableOrders, newSelection)
    currentSelection = newSelection
    currentOrders = availableOrders
	commonGroup:ClearChildren()
	altGroup:ClearChildren()
    CreateButtons(currentOrders)
    CreateAltButtons(currentOrders)
	RadialBG:Hide()

end


function CreateRadialGroup(parent)
    CreateRadialBG(parent)
    commonGroup = Group(RadialBG)
    commonGroup.Width:Set(0)
    commonGroup.Height:Set(0)
    LayoutHelpers.AtCenterIn(commonGroup, RadialBG)
    commonGroup.Depth:Set(function() return RadialBG.Depth() + 10 end)
    altGroup = Group(RadialBG)
    altGroup.Width:Set(0)
    altGroup.Height:Set(0)
    LayoutHelpers.AtCenterIn(altGroup, RadialBG)
    altGroup.Depth:Set(function() return RadialBG.Depth() + 10 end)
	CreateButtons(currentOrders)
	CreateAltButtons(currentOrders)
end


function SetFaction(factionIndex)
    if factionTable[factionIndex] then
	    Faction = factionTable[factionIndex]
	else
	    Faction = 'GENERIC'
	end
end


function CreateGroup(parent, Xpos, Ypos)
        CreateRadialGroup(parent)
		RadialBG:Hide()
		--courtesy of saya :)
		if oldHandleEvent == false then
            oldHandleEvent = import('/lua/modules/ui/game/gamemain.lua').GetGameParent().HandleEvent
		end
		import('/lua/modules/ui/game/gamemain.lua').GetGameParent().HandleEvent = function(self, event)
			if event.Type=="ButtonPress" and event.Modifiers.Middle then 
				if RadialBG ~= nil then
	                if event.Modifiers.Ctrl == true and event.Modifiers.Alt == nil then
					    RadialBG.Left:Set(event.MouseX-80)
					    RadialBG.Top:Set(event.MouseY-80)
	    			    if commonGroup:IsHidden() then
		    			    RadialBG:Show()
						    altGroup:Hide()
			    		else
				    	    RadialBG:Hide()
			    		end
					elseif event.Modifiers.Alt == true and event.Modifiers.Ctrl == nil then
					    RadialBG.Left:Set(event.MouseX-80)
					    RadialBG.Top:Set(event.MouseY-80)
	    			    if altGroup:IsHidden() then
						    RadialBG:Show()
					        commonGroup:Hide()
			    		else
				    	    RadialBG:Hide()
			    		end
					elseif event.Modifiers.Alt == nil and event.Modifiers.Ctrl == nil then
					    RadialBG:Hide()
                    end
				end
			end
			oldHandleEvent(self,event)
		end

end
